Package: cohortBuilder
Type: Package
Title: Data Source Agnostic Filtering Tools
Version: 0.2.0
Authors@R: 
    c(person('Krystian', 'Igras', 
             email = 'krystian8207@gmail.com', 
             role = c('cre', 'aut')),
      person('Adam', 'Foryś',
             role = "ctb"))
Maintainer: Krystian Igras <krystian8207@gmail.com>
Description: Common API for filtering data stored in different data models.
    Provides multiple filter types and reproducible R code.
    Works standalone or with 'shinyCohortBuilder' as the GUI for interactive Shiny apps.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: R6, jsonlite, purrr, tibble, dplyr (>= 1.0.0), tidyr,
        magrittr, glue, ggplot2, rlang (>= 1.0), formatR
KeepSource: true
RoxygenNote: 7.2.3
Suggests: testthat (>= 3.0.0), shiny, knitr, rmarkdown
Config/testthat/edition: 3
Collate: 'cohortBuilder-package.R' 'cohort_methods.R'
        'source_methods.R' 'step.R' 'filter.R' 'attrition.R'
        'bind_keys.R' 'hooks.R' 'list_operators.R' 'repro_code_utils.R'
        'source_tblist.R' 'data.R'
VignetteBuilder: knitr
URL: https://github.com/r-world-devs/cohortBuilder/,
        https://r-world-devs.github.io/cohortBuilder/
Depends: R (>= 3.5.0)
NeedsCompilation: no
Packaged: 2023-02-28 13:46:51 UTC; krystiai
Author: Krystian Igras [cre, aut],
  Adam Foryś [ctb]
Repository: CRAN
Date/Publication: 2023-02-28 17:02:33 UTC
Built: R 4.2.0; ; 2023-07-11 01:14:30 UTC; unix
