# Generated by roxygen2: do not edit by hand

S3method(.collect_data,default)
S3method(.collect_data,tblist)
S3method(.get_attrition_count,default)
S3method(.get_attrition_count,tblist)
S3method(.get_attrition_label,default)
S3method(.get_attrition_label,tblist)
S3method(.get_stats,default)
S3method(.get_stats,tblist)
S3method(.init_step,default)
S3method(.init_step,tblist)
S3method(.post_binding,default)
S3method(.post_filtering,default)
S3method(.pre_filtering,default)
S3method(.pre_filtering,tblist)
S3method(.repro_code_tweak,tblist)
S3method(.run_binding,default)
S3method(.run_binding,tblist)
S3method(add_filter,Cohort)
S3method(add_filter,Source)
S3method(add_step,Cohort)
S3method(add_step,Source)
S3method(as.tblist,data.frame)
S3method(as.tblist,list)
S3method(cb_filter.date_range,tblist)
S3method(cb_filter.discrete,tblist)
S3method(cb_filter.discrete_text,tblist)
S3method(cb_filter.multi_discrete,tblist)
S3method(cb_filter.range,tblist)
S3method(filter,character)
S3method(filter,date_range)
S3method(filter,discrete)
S3method(filter,discrete_text)
S3method(filter,multi_discrete)
S3method(filter,range)
S3method(rm_filter,Cohort)
S3method(rm_filter,Source)
S3method(rm_step,Cohort)
S3method(rm_step,Source)
S3method(set_source,tblist)
S3method(update_filter,Cohort)
S3method(update_filter,Source)
export("%->%")
export(.as_constructor)
export(.collect_data)
export(.gen_id)
export(.get_attrition_count)
export(.get_attrition_label)
export(.get_item)
export(.get_method)
export(.get_stats)
export(.if_value)
export(.init_step)
export(.post_binding)
export(.post_filtering)
export(.pre_filtering)
export(.repro_code_tweak)
export(.run_binding)
export(Source)
export(add_filter)
export(add_hook)
export(add_source)
export(add_step)
export(as.tblist)
export(attrition)
export(bind_key)
export(bind_keys)
export(cb_filter.date_range)
export(cb_filter.discrete)
export(cb_filter.discrete_text)
export(cb_filter.multi_discrete)
export(cb_filter.range)
export(code)
export(cohort)
export(data_key)
export(def_filter)
export(description)
export(filter)
export(get_data)
export(get_hook)
export(get_state)
export(new_filter)
export(plot_data)
export(primary_keys)
export(restore)
export(rm_filter)
export(rm_step)
export(run)
export(set_source)
export(stat)
export(step)
export(sum_up)
export(tblist)
export(update_filter)
export(update_source)
importFrom(dplyr,sym)
importFrom(magrittr,"%>%")
